% initMRF.m
% modified from Egon's rmNodesFromCoeffs.m
% input:  the matrix showing which numbered image segments are
%         connected to which other nodes (neighborMat).  
%         The likelihood for each segment for each disparity (lhoods).
% output: nodes structure, ready to send into propNodes

function [nodes] = initMRF(N_pixels, M_pixels, K_states, prop_mat_beta);
fprintf(1, 'Entering initMRF \n'); tic;

nNodes = N_pixels * M_pixels;

%% - Now init the nodes and the bVectors:
%% -----------------------------------------------

nodes = cell(nNodes, 1);
for n = 1:nNodes
    % initialize belief to equal uncertainty.
    nodes{n}.marginal = ones(K_states,1) / K_states;
    % create links - - - - - - - - - - - - - - - - - - - 
    links = getRasterNeighbors(n, N_pixels, M_pixels);
    nodes{n}.nLinks = length(links);
    nodes{n}.links  = cell(length(links), 1);
    for w = 1:length(links)
        nodes{n}.links{w}.farsIndex = links(w);
        % for generality, specify a propMat for each link and direction
        nodes{n}.links{w}.propMat = getPropMat(K_states, prop_mat_beta);
    end
end

%------------------------------------------------------------------
% now go through every link, and set the links back and 
% check that each link out has exactly one link back.
%------------------------------------------------------------------

fprintf(1,'Checking bidirectionality ... \n');

for n = 1:nNodes
    
    for li = 1:(nodes{n}.nLinks)
        farIdx = nodes{n}.links{li}.farsIndex;
        nhits = 0;
        for farslink = 1:nodes{farIdx}.nLinks
            if (nodes{farIdx}.links{farslink}.farsIndex == n)
                nodes{n}.links{li}.farsLinkToMe = farslink;
                nhits = nhits + 1;
            end
        end
        if (nhits~= 1)
            keyboard
            error('nhits <> 0 in initNodes');
        end
    end
end

fprintf(1,'Done initting nodes: %f secs\n',toc);
