% initNodes.m
% modified from Egon's rmNodesFromCoeffs.m
% input:  the matrix showing which numbered image segments are
%         connected to which other nodes (neighborMat).  
%         The likelihood for each segment for each disparity (lhoods).
% output: nodes structure, ready to send into propNodes


function nodes = initNodes (neighborMat, lhoods)
fprintf(1, 'Entering initNodes \n'); tic;

%%  Total number of nodes across all layers:
%% -----------------------------------------------

nNodes = size(neighborMat,1);

%% - Now init the nodes and the bVectors:
%% -----------------------------------------------

nodes = cell(nNodes, 1);
for n = 1:nNodes
    % create links - - - - - - - - - - - - - - - - - - - 
    links = find(neighborMat(:,n));
    nodes{n}.nLinks = length(links);
    nodes{n}.links  = cell(length(links), 1);
    for w = 1:length(links)
        nodes{n}.links{w}.farsIndex             = links(w);
        nodes{n}.links{w}.farsMessageToMe       = [];
        nodes{n}.links{w}.fars_NEXT_MessageToMe = [];        
    end
    % initialize local evidence - - - - - - - - - - - - - - - - - - - 
    nodes{n}.localEvidence = lhoods(n,:);
    
    fprintf(1, '.');
end
fprintf(1, '\n');


%------------------------------------------------------------------
% now go through every link, and set the links back and 
% check that each link out has exactly one link back.
%------------------------------------------------------------------

fprintf(1,'Checking bidirectionality ... \n');

for n = 1:nNodes
    
    for li = 1:(nodes{n}.nLinks)
        farIdx = nodes{n}.links{li}.farsIndex;
        nhits = 0;
        for farslink = 1:nodes{farIdx}.nLinks
            if (nodes{farIdx}.links{farslink}.farsIndex == n)
                nodes{n}.links{li}.farsLinkToMe = farslink;
                nhits = nhits + 1;
            end
        end
        if (nhits~= 1)
            keyboard
            error('nhits <> 0 in initNodes');
        end
    end
end

fprintf(1,'Done initting nodes: %f secs\n',toc);
